# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-05 06:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import jsonfield.fields
import cq.genuuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.CharField(default=cq.genuuid.genuuid, max_length=128, primary_key=True, serialize=False)),
                ('ts', models.DateTimeField(default=django.utils.timezone.now)),
                ('name', models.CharField(db_index=True, max_length=128)),
                ('aggregate_id', models.CharField(db_index=True, max_length=128)),
                ('data', jsonfield.fields.JSONField(null=True)),
            ],
        ),
        migrations.CreateModel(
            name='UniqueItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namespace', models.CharField(max_length=128)),
                ('value', models.CharField(max_length=255)),
                ('aggregate_id', models.CharField(max_length=128)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='uniqueitem',
            unique_together=set([('namespace', 'value')]),
        ),
    ]
